`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 		 http://www.george-smart.co.uk/FPGA
// Engineer: 		 George Smart, MEng.
// 
// Create Date:    00:42:33 01/AUG/2011 BST
// Publised Date:  03:13:06 02/AUG/2011 BST
// Design Name:    VGA Testing Code
// Module Name:    GS_VGA 
// Project Name:   VGA Testing Code, trending Pong.
// Target Device:  XC3S500E
// Tool versions:  Xilinx ISE 12.1
// Description: 	 Pong Game based on http://www.fpga4fun.com/PongGame.html with some extra
//						 little bits and pieces to help me learn!
//
// Dependencies: 	 Depends syncgen from VidSync_Gen.v (GS) (Modified hsync - fpga4fun)
//
// Revision: 0.4 - Added a few features
// Revision 0.01 - File Created
//
// Additional Comments: 
//    This code was written to help me learn how to program Verilog. It was my first project!
//    Please excuse me if this code is appauling, I know no better!
//
//    This code is based on that of http://www.fpga4fun.com/PongGame.html
//      For which I am grateful!
//
//////////////////////////////////////////////////////////////////////////////////
module GS_VGA(
    output VGA_RED,
    output VGA_GREEN,
    output VGA_BLUE,
    output VGA_HSYNC,
    output VGA_VSYNC,
	 output LED_0,
	 output LED_1,
	 output LED_2,
	 output LED_3,
	 output LED_4,
	 output LED_5,
	 output LED_6,
	 output LED_7,
	 input  ROT_A,
	 input  ROT_B,
	 input  SW_0,
	 input  SW_1,
	 input  SW_2,
	 input  SW_3,
	 input  CLK_50MHZ
    );

// My development board had a 50MHz clock.
// The original code was for a board with a 25MHz clock.
// This section divides CLK_50MHZ by 2 and outputs clk.
reg clk;
always @(posedge CLK_50MHZ)
clk <= ~clk;

wire inDisplayArea;
wire [9:0] CounterX;
wire [8:0] CounterY;

VidSync_Gen syncgen(.clk(clk), .VGA_HSYNC(VGA_HSYNC), .VGA_VSYNC(VGA_VSYNC), 
  .inDisplayArea(inDisplayArea), .CounterX(CounterX), .CounterY(CounterY));

/////////////////////////////////////////////////////////////////
reg [9:0] PaddlePosition, AutoPaddlePosition;
reg [2:0] quadAr, quadBr;
always @(posedge clk) quadAr <= {quadAr[1:0], ROT_A};
always @(posedge clk) quadBr <= {quadBr[1:0], ROT_B};

always @(posedge clk)
if(quadAr[2] ^ quadAr[1] ^ quadBr[2] ^ quadBr[1])
begin
	if(quadAr[2] ^ quadBr[1])
	begin
		if(~&PaddlePosition)        // make sure the value doesn't overflow
			PaddlePosition <= PaddlePosition + 1;
	end
	else
	begin
		if(|PaddlePosition)        // make sure the value doesn't underflow
			PaddlePosition <= PaddlePosition - 1;
	end
end

/////////////////////////////////////////////////////////////////
reg [9:0] ballX;
reg [8:0] ballY;
reg ball_inX, ball_inY;

always @(posedge clk)
if(ball_inX==0) ball_inX <= (CounterX==ballX) & ball_inY; else ball_inX <= !(CounterX==ballX+16);

always @(posedge clk)
if(ball_inY==0) ball_inY <= (CounterY==ballY); else ball_inY <= !(CounterY==ballY+16);

wire ball = ball_inX & ball_inY;

/////////////////////////////////////////////////////////////////
// These sections were modified by GS to A) better fit my screen and B) provide automatic paddle control.
// Draw a border around the screen                //79
wire border = (CounterX[9:3]==0) || (CounterX[9:3]==72) || (CounterY[8:3]==0) || (CounterY[8:3]==59);
wire manualpaddle = (CounterX>=PaddlePosition+8) && (CounterX<=PaddlePosition+120) && (CounterY[8:4]==27);
wire autopaddle = (CounterX>=(AutoPaddlePosition-60)+8) && (CounterX<=(AutoPaddlePosition-60)+120) && (CounterY[8:4]==27);
wire paddle = (SW_0 ? (autopaddle) : manualpaddle); // select based on SW_0
wire BouncingObject = border | paddle; // active if the border or paddle is redrawing itself

reg ball_dirX, ball_dirY;
reg ResetCollision;
always @(posedge clk) ResetCollision <= (CounterY==500) & (CounterX==0);  // active only once for every video frame

reg CollisionX1, CollisionX2, CollisionY1, CollisionY2;
always @(posedge clk) if(ResetCollision) CollisionX1<=0; else if(BouncingObject & (CounterX==ballX   ) & (CounterY==ballY+ 8)) CollisionX1<=1;
always @(posedge clk) if(ResetCollision) CollisionX2<=0; else if(BouncingObject & (CounterX==ballX+16) & (CounterY==ballY+ 8)) CollisionX2<=1;
always @(posedge clk) if(ResetCollision) CollisionY1<=0; else if(BouncingObject & (CounterX==ballX+ 8) & (CounterY==ballY   )) CollisionY1<=1;
always @(posedge clk) if(ResetCollision) CollisionY2<=0; else if(BouncingObject & (CounterX==ballX+ 8) & (CounterY==ballY+16)) CollisionY2<=1;

// Output assigns for LEDs
assign LED_0 = CollisionX1;
assign LED_1 = CollisionX2;
assign LED_2 = CollisionY1;
assign LED_3 = CollisionY2;
assign LED_7 = ROT_A;
assign LED_6 = ROT_B;
assign LED_5 = ball_dirX;
assign LED_4 = ball_dirY;

/////////////////////////////////////////////////////////////////
wire UpdateBallPosition = ResetCollision;  // update the ball position at the same time that we reset the collision detectors

always @(posedge clk)
if(UpdateBallPosition & SW_3) // only update ball if SW_3 allows.
begin
	if(~(CollisionX1 & CollisionX2))        // if collision on both X-sides, don't move in the X direction
	begin
		ballX <= ballX + (ball_dirX ? (SW_1 ? -3 : -1) : (SW_1 ? 3 : 1));  // Speed set by SW_1
		if(CollisionX2) ball_dirX <= 1; else if(CollisionX1) ball_dirX <= 0;
	end

	if(~(CollisionY1 & CollisionY2))        // if collision on both Y-sides, don't move in the Y direction
	begin
		ballY <= ballY + (ball_dirY ? (SW_1 ? -3 : -1) : (SW_1 ? 3 : 1)); // Speed set by SW_1
		if(CollisionY2) ball_dirY <= 1; else if(CollisionY1) ball_dirY <= 0;
	end
	
	AutoPaddlePosition <= ((ballX<=60) ? 60 : (ballX>=520) ? 520 : ballX);  // Don't let autopaddle drive off screen edge!
end 

/////////////////////////////////////////////////////////////////

// Set colour scheme based on SW_2
wire R = ((~SW_2 & (border | paddle | ball)) | (SW_2 & (border | paddle)));
wire G = ((~SW_2 & (border | paddle | ball)) | (SW_2 & (border)));
wire B = ((~SW_2 & (border | paddle | ball)) | (SW_2 & (ball)));

reg VGA_R, VGA_G, VGA_B;

assign VGA_RED   = VGA_R;
assign VGA_GREEN = VGA_G;
assign VGA_BLUE  = VGA_B;

always @(posedge clk)
begin
  VGA_R <= R & inDisplayArea;
  VGA_G <= G & inDisplayArea;
  VGA_B <= B & inDisplayArea;
end

endmodule